package sun.security.provider;

import java.security.UnrecoverableEntryException;
import java.security.KeyStore;
import sun.security.pkcs12.PKCS12KeyStore;
import java.util.Locale;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.io.ByteArrayInputStream;
import sun.misc.IOUtils;
import java.security.cert.CertificateFactory;
import java.security.DigestInputStream;
import java.io.DataInputStream;
import java.io.InputStream;
import java.security.cert.CertificateException;
import java.security.MessageDigest;
import java.io.DataOutputStream;
import java.security.DigestOutputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.security.KeyStoreException;
import java.security.PrivateKey;
import java.util.Date;
import java.security.cert.Certificate;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;
import java.io.IOException;
import java.security.UnrecoverableKeyException;
import sun.security.pkcs.EncryptedPrivateKeyInfo;
import java.security.Key;
import java.util.Hashtable;
import java.security.KeyStoreSpi;

abstract class JavaKeyStore extends KeyStoreSpi
{
    private static final int MAGIC = -17957139;
    private static final int VERSION_1 = 1;
    private static final int VERSION_2 = 2;
    private final Hashtable<String, Object> entries;
    
    JavaKeyStore() {
        this.entries = new Hashtable();
    }
    
    abstract String convertAlias(String p0);
    
    @Override
    public Key engineGetKey(String s, char[] array) throws NoSuchAlgorithmException, UnrecoverableKeyException {
        Object value = this.entries.get(this.convertAlias(s));
        if (value == null || !(value instanceof JavaKeyStore.KeyEntry)) {
            return null;
        }
        if (array == null) {
            throw new UnrecoverableKeyException("Password must not be null");
        }
        byte[] convertToBytes = this.convertToBytes(array);
        KeyProtector keyProtector = new KeyProtector(convertToBytes);
        byte[] protectedPrivKey = ((JavaKeyStore.KeyEntry)value).protectedPrivKey;
        try {
            return keyProtector.recover(new EncryptedPrivateKeyInfo(protectedPrivKey));
        }
        catch (IOException ex) {
            throw new UnrecoverableKeyException("Private key not stored as PKCS #8 EncryptedPrivateKeyInfo");
        }
        finally {
            Arrays.fill(convertToBytes, (byte)0);
        }
    }
    
    @Override
    public Certificate[] engineGetCertificateChain(String s) {
        Object value = this.entries.get(this.convertAlias(s));
        if (value != null && value instanceof JavaKeyStore.KeyEntry) {
            if (((JavaKeyStore.KeyEntry)value).chain == null) {
                return null;
            }
            return (Certificate[])((JavaKeyStore.KeyEntry)value).chain.clone();
        }
        else {
            return null;
        }
    }
    
    @Override
    public Certificate engineGetCertificate(String s) {
        Object value = this.entries.get(this.convertAlias(s));
        if (value != null) {
            if ((value instanceof JavaKeyStore.TrustedCertEntry)) {
                return ((JavaKeyStore.TrustedCertEntry)value).cert;
            }
            if (((JavaKeyStore.KeyEntry)value).chain == null) {
                return null;
            }
            return ((JavaKeyStore.KeyEntry)value).chain[0];
        }
        else {
            return null;
        }
    }
    
    @Override
    public Date engineGetCreationDate(String s) {
        Object value = this.entries.get(this.convertAlias(s));
        if (value != null) {
            if ((value instanceof JavaKeyStore.TrustedCertEntry)) {
                return new Date(((JavaKeyStore.TrustedCertEntry)value).date.getTime());
            }
            return new Date(((JavaKeyStore.KeyEntry)value).date.getTime());
        }
        else {
            return null;
        }
    }
    
    @Override
    public void engineSetKeyEntry(String s, Key key, char[] array, Certificate[] array2) throws KeyStoreException {
        byte[] convertToBytes = null;
        if (!(key instanceof PrivateKey)) {
            throw new KeyStoreException("Cannot store non-PrivateKeys");
        }
        try {
            synchronized (this.entries) {
                JavaKeyStore.KeyEntry value = new JavaKeyStore.KeyEntry();
                value.date = new Date();
                convertToBytes = this.convertToBytes(array);
                value.protectedPrivKey = new KeyProtector(convertToBytes).protect(key);
                if (array2 != null && array2.length != 0) {
                    value.chain = (Certificate[])array2.clone();
                }
                else {
                    value.chain = null;
                }
                this.entries.put(this.convertAlias(s), value);
            }
        }
        catch (NoSuchAlgorithmException ex) {
            throw new KeyStoreException("Key protection algorithm not found");
        }
        finally {
            if (convertToBytes != null) {
                Arrays.fill(convertToBytes, (byte)0);
            }
        }
        return;
    }
    
    @Override
    public void engineSetKeyEntry(String s, byte[] encoded, Certificate[] array) throws KeyStoreException {
        synchronized (this.entries) {
            try {
                new EncryptedPrivateKeyInfo(encoded);
            }
            catch (IOException ex) {
                throw new KeyStoreException("key is not encoded as EncryptedPrivateKeyInfo");
            }
            JavaKeyStore.KeyEntry value = new JavaKeyStore.KeyEntry();
            value.date = new Date();
            value.protectedPrivKey = (byte[])encoded.clone();
            if (array != null && array.length != 0) {
                value.chain = (Certificate[])array.clone();
            }
            else {
                value.chain = null;
            }
            this.entries.put(this.convertAlias(s), value);
        }
    }
    
    @Override
    public void engineSetCertificateEntry(String s, Certificate cert) throws KeyStoreException {
        synchronized (this.entries) {
            Object value = this.entries.get(this.convertAlias(s));
            if (value != null && value instanceof JavaKeyStore.KeyEntry) {
                throw new KeyStoreException("Cannot overwrite own certificate");
            }
            JavaKeyStore.TrustedCertEntry value2 = new JavaKeyStore.TrustedCertEntry();
            value2.cert = cert;
            value2.date = new Date();
            this.entries.put(this.convertAlias(s), value2);
        }
    }
    
    @Override
    public void engineDeleteEntry(String s) throws KeyStoreException {
        synchronized (this.entries) {
            this.entries.remove(this.convertAlias(s));
        }
    }
    
    @Override
    public Enumeration<String> engineAliases() {
        return this.entries.keys();
    }
    
    @Override
    public boolean engineContainsAlias(String s) {
        return this.entries.containsKey(this.convertAlias(s));
    }
    
    @Override
    public int engineSize() {
        return this.entries.size();
    }
    
    @Override
    public boolean engineIsKeyEntry(String s) {
        Object value = this.entries.get(this.convertAlias(s));
        return value != null && value instanceof JavaKeyStore.KeyEntry;
    }
    
    @Override
    public boolean engineIsCertificateEntry(String s) {
        Object value = this.entries.get(this.convertAlias(s));
        return value != null && value instanceof JavaKeyStore.TrustedCertEntry;
    }
    
    @Override
    public String engineGetCertificateAlias(Certificate other) {
        Enumeration<String> keys = this.entries.keys();
        while (keys.hasMoreElements()) {
            String key = (String)keys.nextElement();
            Object value = this.entries.get(key);
            Certificate cert;
            if ((value instanceof JavaKeyStore.TrustedCertEntry)) {
                cert = ((JavaKeyStore.TrustedCertEntry)value).cert;
            }
            else {
                if (((JavaKeyStore.KeyEntry)value).chain == null) {
                    continue;
                }
                cert = ((JavaKeyStore.KeyEntry)value).chain[0];
            }
            if (cert.equals(other)) {
                return key;
            }
            continue;
        }
        return null;
    }
    
    @Override
    public void engineStore(OutputStream stream, char[] array) throws IOException, NoSuchAlgorithmException, CertificateException {
        synchronized (this.entries) {
            if (array == null) {
                throw new IllegalArgumentException("password can't be null");
            }
            MessageDigest preKeyedHash = this.getPreKeyedHash(array);
            DataOutputStream dataOutputStream = new DataOutputStream(new DigestOutputStream(stream, preKeyedHash));
            dataOutputStream.writeInt(-17957139);
            dataOutputStream.writeInt(2);
            dataOutputStream.writeInt(this.entries.size());
            Enumeration<String> keys = this.entries.keys();
            while (keys.hasMoreElements()) {
                String str = (String)keys.nextElement();
                Object value = this.entries.get(str);
                if ((value instanceof JavaKeyStore.KeyEntry)) {
                    dataOutputStream.writeInt(1);
                    dataOutputStream.writeUTF(str);
                    dataOutputStream.writeLong(((JavaKeyStore.KeyEntry)value).date.getTime());
                    dataOutputStream.writeInt(((JavaKeyStore.KeyEntry)value).protectedPrivKey.length);
                    dataOutputStream.write(((JavaKeyStore.KeyEntry)value).protectedPrivKey);
                    int length;
                    if (((JavaKeyStore.KeyEntry)value).chain == null) {
                        length = 0;
                    }
                    else {
                        length = ((JavaKeyStore.KeyEntry)value).chain.length;
                    }
                    dataOutputStream.writeInt(length);
                    for (int i = 0; i < length; ++i) {
                        byte[] encoded2 = ((JavaKeyStore.KeyEntry)value).chain[i].getEncoded();
                        dataOutputStream.writeUTF(((JavaKeyStore.KeyEntry)value).chain[i].getType());
                        dataOutputStream.writeInt(encoded2.length);
                        dataOutputStream.write(encoded2);
                    }
                }
                else {
                    dataOutputStream.writeInt(2);
                    dataOutputStream.writeUTF(str);
                    dataOutputStream.writeLong(((JavaKeyStore.TrustedCertEntry)value).date.getTime());
                    byte[] encoded = ((JavaKeyStore.TrustedCertEntry)value).cert.getEncoded();
                    dataOutputStream.writeUTF(((JavaKeyStore.TrustedCertEntry)value).cert.getType());
                    dataOutputStream.writeInt(encoded.length);
                    dataOutputStream.write(encoded);
                }
            }
            dataOutputStream.write(preKeyedHash.digest());
            dataOutputStream.flush();
        }
    }
    
    @Override
    public void engineLoad(InputStream inputStream, char[] array) throws IOException, NoSuchAlgorithmException, CertificateException {
        synchronized (this.entries) {
            MessageDigest preKeyedHash = null;
            CertificateFactory certificateFactory = null;
            Object o = null;
            if (inputStream == null) {
                return;
            }
            DataInputStream dataInputStream;
            if (array != null) {
                preKeyedHash = this.getPreKeyedHash(array);
                dataInputStream = new DataInputStream(new DigestInputStream(inputStream, preKeyedHash));
            }
            else {
                dataInputStream = new DataInputStream(inputStream);
            }
            int int1 = dataInputStream.readInt();
            int int2 = dataInputStream.readInt();
            if (int1 != -17957139 || (int2 != 1 && int2 != 2)) {
                throw new IOException("Invalid keystore format");
            }
            if (int2 == 1) {
                certificateFactory = CertificateFactory.getInstance("X509");
            }
            else {
                o = new Hashtable(3);
            }
            this.entries.clear();
            for (int int3 = dataInputStream.readInt(), i = 0; i < int3; ++i) {
                int int4 = dataInputStream.readInt();
                if (int4 == 1) {
                    JavaKeyStore.KeyEntry value2 = new JavaKeyStore.KeyEntry();
                    String utf3 = dataInputStream.readUTF();
                    value2.date = new Date(dataInputStream.readLong());
                    value2.protectedPrivKey = IOUtils.readExactlyNBytes(dataInputStream, dataInputStream.readInt());
                    int int5 = dataInputStream.readInt();
                    if (int5 > 0) {
                        ArrayList o2 = new ArrayList((int5 > 10) ? 10 : int5);
                        for (int j = 0; j < int5; ++j) {
                            if (int2 == 2) {
                                String utf4 = dataInputStream.readUTF();
                                if (o.containsKey(utf4)) {
                                    certificateFactory = (CertificateFactory)o.get(utf4);
                                }
                                else {
                                    certificateFactory = CertificateFactory.getInstance(utf4);
                                    o.put(utf4, certificateFactory);
                                }
                            }
                            ByteArrayInputStream inStream2 = new ByteArrayInputStream(IOUtils.readExactlyNBytes(dataInputStream, dataInputStream.readInt()));
                            o2.add(certificateFactory.generateCertificate(inStream2));
                            inStream2.close();
                        }
                        value2.chain = (Certificate[])o2.toArray(new Certificate[int5]);
                    }
                    this.entries.put(utf3, value2);
                }
                else if (int4 == 2) {
                    JavaKeyStore.TrustedCertEntry value = new JavaKeyStore.TrustedCertEntry();
                    String utf = dataInputStream.readUTF();
                    value.date = new Date(dataInputStream.readLong());
                    if (int2 == 2) {
                        String utf2 = dataInputStream.readUTF();
                        if (o.containsKey(utf2)) {
                            certificateFactory = (CertificateFactory)o.get(utf2);
                        }
                        else {
                            certificateFactory = CertificateFactory.getInstance(utf2);
                            o.put(utf2, certificateFactory);
                        }
                    }
                    ByteArrayInputStream inStream = new ByteArrayInputStream(IOUtils.readExactlyNBytes(dataInputStream, dataInputStream.readInt()));
                    value.cert = certificateFactory.generateCertificate(inStream);
                    inStream.close();
                    this.entries.put(utf, value);
                }
                else {
                    throw new IOException("Unrecognized keystore entry");
                }
            }
            if (array != null) {
                byte[] digest = preKeyedHash.digest();
                if (!MessageDigest.isEqual(digest, IOUtils.readExactlyNBytes(dataInputStream, digest.length))) {
                    throw (IOException)new IOException("Keystore was tampered with, or password was incorrect").initCause(new UnrecoverableKeyException("Password verification failed"));
                }
            }
        }
    }
    
    private MessageDigest getPreKeyedHash(char[] array) throws NoSuchAlgorithmException, UnsupportedEncodingException {
        MessageDigest instance = MessageDigest.getInstance("SHA");
        byte[] convertToBytes = this.convertToBytes(array);
        instance.update(convertToBytes);
        Arrays.fill(convertToBytes, (byte)0);
        instance.update("Mighty Aphrodite".getBytes("UTF8"));
        return instance;
    }
    
    private byte[] convertToBytes(char[] array) {
        byte[] array2 = new byte[array.length * 2];
        int i = 0;
        int n = 0;
        while (i < array.length) {
            array2[n++] = (byte)(array[i] >> 8);
            array2[n++] = (byte)array[i];
            ++i;
        }
        return array2;
    }
    
    public static final class JKS extends JavaKeyStore
    {
        @Override
        String convertAlias(String s) {
            return s.toLowerCase(Locale.ENGLISH);
        }
    }
    
    public static final class CaseExactJKS extends JavaKeyStore
    {
        @Override
        String convertAlias(String s) {
            return s;
        }
    }
    
    public static final class DualFormatJKS extends KeyStoreDelegator
    {
        public DualFormatJKS() {
            super("JKS", JavaKeyStore.JKS.class, "PKCS12", PKCS12KeyStore.class);
        }
    }
    
    private static class KeyEntry
    {
        Date date;
        byte[] protectedPrivKey;
        Certificate[] chain;
    }
    
    private static class TrustedCertEntry
    {
        Date date;
        Certificate cert;
    }
}
